@@ -18,7 +18,7 @@ use crate::cassandra_sys::cass_collection_append_int16;
 use crate::cassandra_sys::cass_collection_append_int32;
 use crate::cassandra_sys::cass_collection_append_int64;
 use crate::cassandra_sys::cass_collection_append_int8;
-use crate::cassandra_sys::cass_collection_append_string;
+use crate::cassandra_sys::cass_collection_append_string_n;
 use crate::cassandra_sys::cass_collection_append_tuple;
 use crate::cassandra_sys::cass_collection_append_uint32;
 use crate::cassandra_sys::cass_collection_append_user_type;
@@ -34,7 +34,7 @@ use crate::cassandra_sys::CASS_COLLECTION_TYPE_LIST;
 use crate::cassandra_sys::CASS_COLLECTION_TYPE_MAP;
 use crate::cassandra_sys::CASS_COLLECTION_TYPE_SET;
 
-use std::ffi::CString;
+use std::os::raw::c_char;
 
 // #[repr(C)]
 // #[derive(Debug,Copy,Clone)]
@@ -45,12 +45,21 @@ use std::ffi::CString;
 // }
 
 /// A generic Cassandra collection that needs to go away
-pub trait CassCollection {
+pub trait CassCollection: Sized {
     /// The type of value held by this collection
     type Value;
 
     /// Creates a new collection.
-    fn new(item_count: usize) -> Self;
+    fn new() -> Self {
+        Self::with_capacity(0)
+    }
+
+    /// Creates a new collection, specifying a capacity which will be
+    /// used as a size hint to avoid re-allocations of the collection internally.
+    ///
+    /// If you know the number of items you'll insert into the collection ahead of time,
+    /// it is recommended to use this function over [`CassCollection::new`].
+    fn with_capacity(capacity: usize) -> Self;
 
     /// Creates a new collection from an existing data type.
     fn new_from_data_type(value: DataType, item_count: usize) -> Self;
@@ -165,8 +174,8 @@ impl CassCollection for List {
     type Value = _CassCollection;
 
     /// create a new list
-    fn new(item_count: usize) -> Self {
-        unsafe { List::build(cass_collection_new(CASS_COLLECTION_TYPE_LIST, item_count)) }
+    fn with_capacity(capacity: usize) -> Self {
+        unsafe { List::build(cass_collection_new(CASS_COLLECTION_TYPE_LIST, capacity)) }
     }
 
     fn new_from_data_type(value: DataType, item_count: usize) -> Self {
@@ -230,8 +239,8 @@ impl CassCollection for List {
     /// Appends an "ascii", "text" or "varchar" to the collection.
     fn append_string(&mut self, value: &str) -> Result<&mut Self> {
         unsafe {
-            let cstr = CString::new(value)?;
-            let result = cass_collection_append_string(self.inner(), cstr.as_ptr());
+            let value_ptr = value.as_ptr() as *const c_char;
+            let result = cass_collection_append_string_n(self.inner(), value_ptr, value.len());
             result.to_result(self)
         }
     }
@@ -302,8 +311,8 @@ impl CassCollection for Set {
     type Value = _CassCollection;
 
     /// create a new list
-    fn new(item_count: usize) -> Self {
-        unsafe { Set(cass_collection_new(CASS_COLLECTION_TYPE_SET, item_count)) }
+    fn with_capacity(capacity: usize) -> Self {
+        unsafe { Set(cass_collection_new(CASS_COLLECTION_TYPE_SET, capacity)) }
     }
 
     fn new_from_data_type(value: DataType, item_count: usize) -> Self {
@@ -366,8 +375,8 @@ impl CassCollection for Set {
     /// Appends an "ascii", "text" or "varchar" to the collection.
     fn append_string(&mut self, value: &str) -> Result<&mut Self> {
         unsafe {
-            let cstr = CString::new(value)?;
-            let result = cass_collection_append_string(self.inner(), cstr.as_ptr());
+            let value_ptr = value.as_ptr() as *const c_char;
+            let result = cass_collection_append_string_n(self.inner(), value_ptr, value.len());
             result.to_result(self)
         }
     }
@@ -432,9 +441,9 @@ impl Drop for Map {
 
 impl CassCollection for Map {
     type Value = _CassCollection;
-    /// create a new list
-    fn new(item_count: usize) -> Self {
-        unsafe { Map(cass_collection_new(CASS_COLLECTION_TYPE_MAP, item_count)) }
+
+    fn with_capacity(capacity: usize) -> Self {
+        unsafe { Map(cass_collection_new(CASS_COLLECTION_TYPE_MAP, capacity)) }
     }
 
     fn new_from_data_type(value: DataType, item_count: usize) -> Self {
@@ -498,8 +507,8 @@ impl CassCollection for Map {
     /// Appends an "ascii", "text" or "varchar" to the collection.
     fn append_string(&mut self, value: &str) -> Result<&mut Self> {
         unsafe {
-            let cstr = CString::new(value)?;
-            let result = cass_collection_append_string(self.inner(), cstr.as_ptr());
+            let value_ptr = value.as_ptr() as *const c_char;
+            let result = cass_collection_append_string_n(self.inner(), value_ptr, value.len());
             result.to_result(self)
         }
     }
