@@ -4,6 +4,7 @@
 
 use crate::cassandra::batch::Batch;
 use crate::cassandra::cluster::Cluster;
+use crate::cassandra::custom_payload::CustomPayloadResponse;
 use crate::cassandra::error::*;
 use crate::cassandra::future::CassFuture;
 use crate::cassandra::metrics::SessionMetrics;
@@ -12,22 +13,23 @@ use crate::cassandra::result::CassResult;
 use crate::cassandra::schema::schema_meta::SchemaMeta;
 use crate::cassandra::statement::Statement;
 use crate::cassandra::util::Protected;
+use crate::CustomPayload;
 
 use crate::cassandra_sys::cass_session_close;
 use crate::cassandra_sys::cass_session_connect;
-use crate::cassandra_sys::cass_session_connect_keyspace;
+use crate::cassandra_sys::cass_session_connect_keyspace_n;
 use crate::cassandra_sys::cass_session_execute;
 use crate::cassandra_sys::cass_session_execute_batch;
 use crate::cassandra_sys::cass_session_free;
 use crate::cassandra_sys::cass_session_get_metrics;
 use crate::cassandra_sys::cass_session_get_schema_meta;
 use crate::cassandra_sys::cass_session_new;
-use crate::cassandra_sys::cass_session_prepare;
+use crate::cassandra_sys::cass_session_prepare_n;
 use crate::cassandra_sys::CassSession as _Session;
 
-use std::ffi::CString;
 use std::ffi::NulError;
 use std::mem;
+use std::os::raw::c_char;
 
 /// A session object is used to execute queries and maintains cluster state through
 /// the control connection. The control connection is used to auto-discover nodes and
@@ -88,11 +90,12 @@ impl Session {
     /// Connects a session and sets the keyspace.
     pub fn connect_keyspace(&self, cluster: &Cluster, keyspace: &str) -> Result<CassFuture<()>> {
         unsafe {
-            let keyspace_cstr = CString::new(keyspace)?;
-            Ok(<CassFuture<()>>::build(cass_session_connect_keyspace(
+            let keyspace_ptr = keyspace.as_ptr() as *const c_char;
+            Ok(<CassFuture<()>>::build(cass_session_connect_keyspace_n(
                 self.0,
                 cluster.inner(),
-                keyspace_cstr.as_ptr(),
+                keyspace_ptr,
+                keyspace.len(),
             )))
         }
     }
@@ -107,9 +110,9 @@ impl Session {
     /// Create a prepared statement.
     pub fn prepare(&self, query: &str) -> Result<CassFuture<PreparedStatement>> {
         unsafe {
-            let query_cstr = CString::new(query)?;
+            let query_ptr = query.as_ptr() as *const c_char;
             Ok(<CassFuture<PreparedStatement>>::build(
-                cass_session_prepare(self.0, query_cstr.as_ptr()),
+                cass_session_prepare_n(self.0, query_ptr, query.len()),
             ))
         }
     }
@@ -129,11 +132,34 @@ impl Session {
         })
     }
 
+    /// Execute a batch statement and get any custom payloads from the response.
+    pub fn execute_batch_with_payloads(
+        &self,
+        batch: Batch,
+    ) -> CassFuture<(CassResult, CustomPayloadResponse)> {
+        <CassFuture<(CassResult, CustomPayloadResponse)>>::build(unsafe {
+            cass_session_execute_batch(self.0, batch.inner())
+        })
+    }
+
     /// Execute a statement.
     pub fn execute(&self, statement: &Statement) -> CassFuture<CassResult> {
         unsafe { <CassFuture<CassResult>>::build(cass_session_execute(self.0, statement.inner())) }
     }
 
+    /// Execute a statement and get any custom payloads from the response.
+    pub fn execute_with_payloads(
+        &self,
+        statement: &Statement,
+    ) -> CassFuture<(CassResult, CustomPayloadResponse)> {
+        unsafe {
+            <CassFuture<(CassResult, CustomPayloadResponse)>>::build(cass_session_execute(
+                self.0,
+                statement.inner(),
+            ))
+        }
+    }
+
     /// Gets a snapshot of this session's schema metadata. The returned
     /// snapshot of the schema metadata is not updated. This function
     /// must be called again to retrieve any schema changes since the
